/**
Utility routines for testing out and timing
vertex and fragment shaders.
*/
/************* 3D Noise Texture ************/
/// Upload random value noise as a 3D texture of this size.
void uploadNoise(int size) {
	int sx=size,sy=size,sz=size;
	unsigned char *noise=new unsigned char[4*sx*sy*sz];
	for (int z=0;z<sz;z++)
	for (int y=0;y<sy;y++)
	for (int x=0;x<sx;x++) {
		unsigned char *n=&noise[4*(x+sx*(y+sy*z))];
		n[0]=0xff&rand(); // RGB all pure noise
		n[1]=0xff&rand();
		n[2]=0xff&rand();
		n[3]=0xff; // alpha is 1
	}
	glTexImage3D(GL_TEXTURE_3D,0,GL_RGBA8,
		sx,sy,sz,0,
		GL_RGBA,GL_UNSIGNED_BYTE,
		noise);
	delete[] noise;
	oglCheck("upload 3d tex");
	
	glTexParameterf(GL_TEXTURE_3D, GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameterf(GL_TEXTURE_3D, GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	oglCheck("linear 3d tex");
	
	glTexParameterf(GL_TEXTURE_3D, GL_TEXTURE_WRAP_S, GL_REPEAT); 
	glTexParameterf(GL_TEXTURE_3D, GL_TEXTURE_WRAP_T, GL_REPEAT); 
	glTexParameterf(GL_TEXTURE_3D, GL_TEXTURE_WRAP_R, GL_REPEAT); 
	oglCheck("repeat 3d tex");
}

/*************** Fragment-Shaded Objects ****************/
void drawFragmentObjects(const oglViewpoint &vp) {
	if (oglToggles['t']) 
	{ /* Just do a timing test */
		double z=0.0;
		
		/// Draw a bunch of tiny triangles, to measure triangle rate
		glFinish();
		double start_tiny=osl::time();
		enum {nTiny=100000};
		for (int t=0;t<nTiny;t++) {
		        glBegin(GL_TRIANGLES);
		        double sz=0.01;
		        double x=-1+0.02*(t%100), y=-1+0.02*(t/10000);
		        glVertex3f(x,y,z);
		        glVertex3f(x,y+sz,z);
		        glVertex3f(x,y+sz,z+sz);
		        glEnd();
		}
		glFinish();
		double per_tiny=(osl::time()-start_tiny)/nTiny;
		
		/// Draw a bunch of screen-filling quads, to measure fill rate
		glFinish();
		double start_big=osl::time();
		enum {nBig=10};
		for (int b=0;b<nBig;b++) {
		        glBegin(GL_QUADS);
		        double sz=20.0;
		        double x=-0.5*sz, y=-0.5*sz;
		        glVertex3f(x,y,z);
		        glVertex3f(x,y+sz,z);
		        glVertex3f(x+sz,y+sz,z);
		        glVertex3f(x+sz,y,z);
		        glEnd();
		}
		glFinish();
		double per_big=(osl::time()-start_big)/nBig;
		double per_pixel=per_big/(vp.getWidth()*vp.getHeight());
		printf("Timing: %.2f ns/triangle,  %.2f ns/pixel (%s)\n",
			1.0e9*per_tiny,  1.0e9*per_pixel,
			oglToggles['f']?"conventional":"fragment shaders");
		
		oglToggles['t']=0;
		oglRepaint(10);
		
		return;
	}
	
	
	if (oglToggles['T']) 
	{ /* Timing demo: */
		int n=33; /* Array of n x n triangles */
		double sz=1.0/n;
		for (int iy=0;iy<n;iy++)
		for (int ix=0;ix<n;ix++) {
		        glBegin(GL_TRIANGLES);
		        double x=ix*sz-0.5, y=iy*sz-0.5;
			double z=0.2*sin(x*x+y*y);
		        glVertex3f(x,y,z);
		        glVertex3f(x,y+sz,z);
		        glVertex3f(x+sz,y+sz,z);
		        glEnd();
		}
		
		oglRepaint(10);
		return;
	}
	
	
	if (oglToggles['a']) 
	{ /* Array of spheres */
		enum {nx=10}; enum {ny=nx}; enum {nz=nx};
		for (int z=0;z<nz;z++)
		for (int y=0;y<ny;y++)
		for (int x=0;x<nx;x++) {
			oglVector3d color(1.0);
			oglVector3d loc(x*(1.0/nx),y*(1.0/ny),z*(1.0/nz));
			loc*=3;
			glPushMatrix();
			glColor3dv((const double *)color);
			glTranslated(loc.x,loc.y,loc.z); 
			glutSolidSphere(0.5/nx,6,4);
			glPopMatrix();
		}
		return;
	}
	if (oglToggles['p']) 
	{ /* Array of planes */
		enum {nz=5};
		for (int z=0;z<nz;z++) {
			double sz=3;
			oglVector3d loc(-0.5*sz,-0.5*sz,z*(sz/(nz-1)));
			oglTextureQuad(loc,loc+sz*oglVector3d(1,0,0),
				loc+sz*oglVector3d(0,1,0),loc+sz*oglVector3d(1,1,0));
		}
		return;
	}
	
	glColor3f(1,1,0); /* big yellow objects */
	
	// Draw a teapot mandala
	for (double radius=1.0; radius<3.0; radius+=10.8)
	for (double ang=0;ang<360;ang+=0.25*360/radius) {
		glPushMatrix();
		glRotated(ang,0,0,1);
		glTranslated(0.7*radius,0,0);
		if (oglToggles['s']) glutSolidSphere(0.4,40,60);
		else glutSolidTeapot(0.5);
		glPopMatrix();
	}
	
	/* Mark X, Y, and Z axes with little spheres */
	for (int axis=0;axis<3;axis++) {
		oglVector3d color(0.0), loc(0.0);
		color[axis]=1.0; loc[axis]=3.0;
		
		glPushMatrix();
		glColor3dv((const double *)color);
		glTranslated(loc.x,loc.y,loc.z); 
		glutSolidSphere(0.3,20,10);
		glPopMatrix();
	}
}


